import { declare } from '@babel/helper-plugin-utils';
import proposalAsyncDoExpressions from '@babel/plugin-proposal-async-do-expressions';
import proposalDestructuringPrivate from '@babel/plugin-proposal-destructuring-private';
import proposalDoExpressions from '@babel/plugin-proposal-do-expressions';
import proposalDuplicateNamedCapturingGroupsRegex from '@babel/plugin-proposal-duplicate-named-capturing-groups-regex';
import proposalFunctionBind from '@babel/plugin-proposal-function-bind';
import proposalFunctionSent from '@babel/plugin-proposal-function-sent';
import proposalImportDefer from '@babel/plugin-proposal-import-defer';
import proposalImportWasmSource from '@babel/plugin-proposal-import-wasm-source';
import proposalOptionalChainingAssign from '@babel/plugin-proposal-optional-chaining-assign';
import proposalPartialApplication from '@babel/plugin-proposal-partial-application';
import proposalPipelineOperator from '@babel/plugin-proposal-pipeline-operator';
import proposalRecordAndTuple from '@babel/plugin-proposal-record-and-tuple';
import proposalRegexpModifiers from '@babel/plugin-proposal-regexp-modifiers';
import proposalThrowExpression from '@babel/plugin-proposal-throw-expressions';
import syntaxDecorators from '@babel/plugin-syntax-decorators';
import syntaxTypeScript from '@babel/plugin-syntax-typescript';
import transformModulesCommonjs from '@babel/plugin-transform-modules-commonjs';

const [major, minor] = process.versions.node.split(".");
const IS_RECORD_TUPLE_SUPPORTED = (
  // eslint-disable-next-line no-magic-numbers
  +major > 14 || +major === 14 && +minor >= 6
);

var syntaxV8intrinsic = declare((api) => {
  api.assertVersion(7);
  return {
    name: "v8intrinsic",
    manipulateOptions(_opts, parserOpts) {
      parserOpts.plugins.push("v8intrinsic");
    }
  };
});

var index = declare(
  (api, {
    decoratorsBeforeExport,
    decoratorsLegacy = true,
    importDefer,
    isTSX,
    pipelineOperator = "minimal",
    optionalChainingAssignVersion = "2023-07",
    recordTuplePolyfill = IS_RECORD_TUPLE_SUPPORTED,
    recordTupleSyntaxType = "hash"
  }) => {
    api.assertVersion(7);
    return {
      plugins: [
        [
          syntaxDecorators,
          {
            decoratorsBeforeExport,
            legacy: decoratorsLegacy
          }
        ],
        [
          syntaxTypeScript,
          {
            isTSX
          }
        ],
        syntaxV8intrinsic,
        proposalAsyncDoExpressions,
        proposalDestructuringPrivate,
        proposalDuplicateNamedCapturingGroupsRegex,
        proposalDoExpressions,
        proposalFunctionBind,
        proposalFunctionSent,
        ...importDefer ? [proposalImportDefer, transformModulesCommonjs] : [],
        proposalPartialApplication,
        proposalImportWasmSource,
        [
          proposalOptionalChainingAssign,
          {
            version: optionalChainingAssignVersion
          }
        ],
        [
          proposalPipelineOperator,
          {
            proposal: pipelineOperator
          }
        ],
        recordTuplePolyfill && [
          proposalRecordAndTuple,
          {
            importPolyfill: !!recordTuplePolyfill,
            polyfillModuleName: typeof recordTuplePolyfill === "string" ? recordTuplePolyfill : void 0,
            syntaxType: recordTupleSyntaxType
          }
        ],
        proposalRegexpModifiers,
        proposalThrowExpression
      ].filter(Boolean),
      // no need to override if it has been enabled
      overrides: isTSX ? void 0 : [
        {
          test: /\.[jt]sx$/,
          plugins: [
            [
              syntaxTypeScript,
              {
                isTSX: true
              }
            ]
          ]
        }
      ]
    };
  }
);

export { index as default };
